@@ -12,11 +12,11 @@ testb *testbvar = new testb;
 struct testc { virtual void a(); };
 inline void testc::a() {}
 
-// Key functions with inline specifier (PR5705)
+// Functions with inline specifier are not key functions (PR5705)
 struct testd { inline virtual void a(); };
 void testd::a() {}
 
-// Key functions with inline specifier (PR5705)
+// Functions with inline specifier are not key functions (PR5705)
 struct teste { inline virtual void a(); };
 teste *testevar = new teste;
 
@@ -32,11 +32,20 @@ namespace {
 }
 testg *testgvar = new testg;
 
+struct X0 { virtual ~X0(); };
+struct X1 : X0 {
+  virtual void f();
+};
+
+inline void X1::f() { }
+
+void use_X1(X1 *x1) { x1->f(); }
+
 // FIXME: The checks are extremely difficult to get right when the globals
 // aren't alphabetized
+// CHECK: @_ZTV2X1 = weak_odr constant
 // CHECK: @_ZTV5testa = constant [3 x i8*] [i8* null
 // CHECK: @_ZTV5testc = weak_odr constant [3 x i8*] [i8* null
 // CHECK: @_ZTVN12_GLOBAL__N_15testgE = internal constant [3 x i8*] [i8* null
 // CHECK: @_ZTV5teste = weak_odr constant [3 x i8*] [i8* null
 // CHECK: @_ZTV5testb = weak_odr constant [3 x i8*] [i8* null
-
