@@ -5,8 +5,6 @@ char *const_cast_test(const char *var)
   return const_cast<char*>(var);
 }
 
-#if 0
-// FIXME: Uncomment when C++ is supported more.
 struct A {
   virtual ~A() {}
 };
@@ -18,7 +16,6 @@ struct B *dynamic_cast_test(struct A *a)
 {
   return dynamic_cast<struct B*>(a);
 }
-#endif
 
 char *reinterpret_cast_test()
 {
@@ -34,3 +31,9 @@ char postfix_expr_test()
 {
   return reinterpret_cast<char*>(0xdeadbeef)[0];
 }
+
+// This was being incorrectly tentatively parsed.
+namespace test1 {
+  template <class T> class A {};
+  void foo() { A<int>(*(A<int>*)0); }
+}
