@@ -7,8 +7,8 @@ protected:
   static int sf(), u;
 
   struct S {};
-  enum {};
-  int; // expected-error {{declaration does not declare anything}}
+  enum {}; // expected-warning{{declaration does not declare anything}}
+  int; // expected-warning {{declaration does not declare anything}}
   int : 1, : 2;
 
 public:
