@@ -5,11 +5,11 @@ typedef int int4 __attribute__((vector_size(16)));
 typedef int4* int4p;
 
 void test1(float4 a, int4 *result, int i) {
-    result[i] = a; // expected-error {{assigning 'float4', expected 'int4'}}
+    result[i] = a; // expected-error {{assigning to 'int4' from incompatible type 'float4'}}
 }
 
 void test2(float4 a, int4p result, int i) {
-    result[i] = a; // expected-error {{assigning 'float4', expected 'int4'}}
+    result[i] = a; // expected-error {{assigning to 'int4' from incompatible type 'float4'}}
 }
 
 // PR2039
