@@ -10,7 +10,7 @@ void f() {
 }
 
 class Y { 
-  explicit Y(float);
+public: explicit Y(float);
 };
 
 class X { // expected-note{{candidate constructor (the implicit copy constructor)}}
@@ -28,11 +28,11 @@ public:
 void g() {
   X x1(5);
   X x2(1.0, 3, 4.2);
-  X x3(1.0, 1.0); // expected-error{{no matching constructor for initialization of 'class X'}}
+  X x3(1.0, 1.0); // expected-error{{no matching constructor for initialization of 'X'}}
   Y y(1.0);
   X x4(3.14, y);
 
-  Z z; // expected-error{{no matching constructor for initialization of 'class Z'}}
+  Z z; // expected-error{{no matching constructor for initialization of 'Z'}}
 }
 
 struct Base {
@@ -44,7 +44,7 @@ struct Derived : Base {
 };
 
 void foo(const Derived cd, Derived d) {
-        int *pi = cd;	// expected-error {{no viable conversion from 'struct Derived const' to 'int *'}}
+        int *pi = cd;	// expected-error {{no viable conversion from 'Derived const' to 'int *'}}
         int *ppi = d; 
 
 }
